@@ -13,45 +13,42 @@
 // MIT/Apache 2.0 license.
 
 use proc_macro2::{Ident, Span, TokenStream};
-use proc_macro_error::{abort, abort_call_site};
 use quote::{format_ident, quote, quote_spanned};
 use syn::{spanned::Spanned, Data, DeriveInput, FieldsUnnamed, Generics, Variant};
 
 use crate::derives::args;
-use crate::dummies;
+use crate::derives::args::collect_args_fields;
 use crate::item::{Item, Kind, Name};
 use crate::utils::{is_simple_ty, subty_if_name};
 
-pub fn derive_subcommand(input: &DeriveInput) -> TokenStream {
+pub(crate) fn derive_subcommand(input: &DeriveInput) -> Result<TokenStream, syn::Error> {
     let ident = &input.ident;
 
-    dummies::subcommand(ident);
-
     match input.data {
         Data::Enum(ref e) => {
             let name = Name::Derived(ident.clone());
-            let item = Item::from_subcommand_enum(input, name);
+            let item = Item::from_subcommand_enum(input, name)?;
             let variants = e
                 .variants
                 .iter()
                 .map(|variant| {
                     let item =
-                        Item::from_subcommand_variant(variant, item.casing(), item.env_casing());
-                    (variant, item)
+                        Item::from_subcommand_variant(variant, item.casing(), item.env_casing())?;
+                    Ok((variant, item))
                 })
-                .collect::<Vec<_>>();
+                .collect::<Result<Vec<_>, syn::Error>>()?;
             gen_for_enum(&item, ident, &input.generics, &variants)
         }
         _ => abort_call_site!("`#[derive(Subcommand)]` only supports enums"),
     }
 }
 
-pub fn gen_for_enum(
+pub(crate) fn gen_for_enum(
     item: &Item,
     item_name: &Ident,
     generics: &Generics,
     variants: &[(&Variant, Item)],
-) -> TokenStream {
+) -> Result<TokenStream, syn::Error> {
     if !matches!(&*item.kind(), Kind::Command(_)) {
         abort! { item.kind().span(),
             "`{}` cannot be used with `command`",
@@ -61,15 +58,21 @@ pub fn gen_for_enum(
 
     let (impl_generics, ty_generics, where_clause) = generics.split_for_impl();
 
-    let from_arg_matches = gen_from_arg_matches(variants);
-    let update_from_arg_matches = gen_update_from_arg_matches(variants);
+    let from_arg_matches = gen_from_arg_matches(variants)?;
+    let update_from_arg_matches = gen_update_from_arg_matches(variants)?;
 
-    let augmentation = gen_augment(variants, item, false);
-    let augmentation_update = gen_augment(variants, item, true);
-    let has_subcommand = gen_has_subcommand(variants);
+    let augmentation = gen_augment(variants, item, false)?;
+    let augmentation_update = gen_augment(variants, item, true)?;
+    let has_subcommand = gen_has_subcommand(variants)?;
 
-    quote! {
-        #[allow(dead_code, unreachable_code, unused_variables, unused_braces)]
+    Ok(quote! {
+        #[allow(
+            dead_code,
+            unreachable_code,
+            unused_variables,
+            unused_braces,
+            unused_qualifications,
+        )]
         #[allow(
             clippy::style,
             clippy::complexity,
@@ -80,8 +83,10 @@ pub fn gen_for_enum(
             clippy::nursery,
             clippy::cargo,
             clippy::suspicious_else_formatting,
+            clippy::almost_swapped,
+            clippy::redundant_locals,
         )]
-        #[deny(clippy::correctness)]
+        #[automatically_derived]
         impl #impl_generics clap::FromArgMatches for #item_name #ty_generics #where_clause {
             fn from_arg_matches(__clap_arg_matches: &clap::ArgMatches) -> ::std::result::Result<Self, clap::Error> {
                 Self::from_arg_matches_mut(&mut __clap_arg_matches.clone())
@@ -95,7 +100,13 @@ pub fn gen_for_enum(
             #update_from_arg_matches
         }
 
-        #[allow(dead_code, unreachable_code, unused_variables, unused_braces)]
+        #[allow(
+            dead_code,
+            unreachable_code,
+            unused_variables,
+            unused_braces,
+            unused_qualifications,
+        )]
         #[allow(
             clippy::style,
             clippy::complexity,
@@ -106,8 +117,10 @@ pub fn gen_for_enum(
             clippy::nursery,
             clippy::cargo,
             clippy::suspicious_else_formatting,
+            clippy::almost_swapped,
+            clippy::redundant_locals,
         )]
-        #[deny(clippy::correctness)]
+        #[automatically_derived]
         impl #impl_generics clap::Subcommand for #item_name #ty_generics #where_clause {
             fn augment_subcommands <'b>(__clap_app: clap::Command) -> clap::Command {
                 #augmentation
@@ -119,217 +132,214 @@ pub fn gen_for_enum(
                 #has_subcommand
             }
         }
-    }
+    })
 }
 
 fn gen_augment(
     variants: &[(&Variant, Item)],
     parent_item: &Item,
     override_required: bool,
-) -> TokenStream {
-    use syn::Fields::*;
+) -> Result<TokenStream, syn::Error> {
+    use syn::Fields::{Named, Unit, Unnamed};
 
     let app_var = Ident::new("__clap_app", Span::call_site());
 
-    let subcommands: Vec<_> = variants
-        .iter()
-        .filter_map(|(variant, item)| {
-            let kind = item.kind();
+    let mut subcommands = Vec::new();
+    for (variant, item) in variants {
+        let kind = item.kind();
 
-            match &*kind {
-                Kind::Skip(_, _) |
-                Kind::Arg(_) |
-                Kind::FromGlobal(_) |
-                Kind::Value => None,
+        let genned = match &*kind {
+            Kind::Skip(_, _) | Kind::Arg(_) | Kind::FromGlobal(_) | Kind::Value => None,
 
-                Kind::ExternalSubcommand => {
-                    let ty = match variant.fields {
-                        Unnamed(ref fields) if fields.unnamed.len() == 1 => &fields.unnamed[0].ty,
+            Kind::ExternalSubcommand => {
+                let ty = match variant.fields {
+                    Unnamed(ref fields) if fields.unnamed.len() == 1 => &fields.unnamed[0].ty,
 
-                        _ => abort!(
-                            variant,
-                            "The enum variant marked with `external_subcommand` must be \
-                             a single-typed tuple, and the type must be either `Vec<String>` \
-                             or `Vec<OsString>`."
-                        ),
-                    };
-                    let deprecations = if !override_required {
-                        item.deprecations()
-                    } else {
-                        quote!()
-                    };
-                    let subty = subty_if_name(ty, "Vec").unwrap_or_else(|| {
-                        abort!(
-                            ty.span(),
-                            "The type must be `Vec<_>` \
-                             to be used with `external_subcommand`."
-                        )
-                    });
-                    let subcommand = quote_spanned! { kind.span()=>
-                        #deprecations
-                        let #app_var = #app_var
-                            .external_subcommand_value_parser(clap::value_parser!(#subty));
-                    };
-                    Some(subcommand)
-                }
-
-                Kind::Flatten(_) => match variant.fields {
-                    Unnamed(FieldsUnnamed { ref unnamed, .. }) if unnamed.len() == 1 => {
-                        let ty = &unnamed[0];
-                        let deprecations = if !override_required {
-                            item.deprecations()
-                        } else {
-                            quote!()
-                        };
-                        let next_help_heading = item.next_help_heading();
-                        let next_display_order = item.next_display_order();
-                        let subcommand = if override_required {
-                            quote! {
-                                #deprecations
-                                let #app_var = #app_var
-                                    #next_help_heading
-                                    #next_display_order;
-                                let #app_var = <#ty as clap::Subcommand>::augment_subcommands_for_update(#app_var);
-                            }
-                        } else {
-                            quote! {
-                                #deprecations
-                                let #app_var = #app_var
-                                    #next_help_heading
-                                    #next_display_order;
-                                let #app_var = <#ty as clap::Subcommand>::augment_subcommands(#app_var);
-                            }
-                        };
-                        Some(subcommand)
-                    }
                     _ => abort!(
                         variant,
-                        "`flatten` is usable only with single-typed tuple variants"
+                        "The enum variant marked with `external_subcommand` must be \
+                             a single-typed tuple, and the type must be either `Vec<String>` \
+                             or `Vec<OsString>`."
                     ),
-                },
+                };
+                let deprecations = if !override_required {
+                    item.deprecations()
+                } else {
+                    quote!()
+                };
+                let subty = subty_if_name(ty, "Vec").ok_or_else(|| {
+                    format_err!(
+                        ty.span(),
+                        "The type must be `Vec<_>` \
+                             to be used with `external_subcommand`."
+                    )
+                })?;
+                let subcommand = quote_spanned! { kind.span()=>
+                    #deprecations
+                    let #app_var = #app_var
+                        .external_subcommand_value_parser(clap::value_parser!(#subty));
+                };
+                Some(subcommand)
+            }
 
-                Kind::Subcommand(_) => {
-                    let subcommand_var = Ident::new("__clap_subcommand", Span::call_site());
-                    let arg_block = match variant.fields {
-                        Named(_) => {
-                            abort!(variant, "non single-typed tuple enums are not supported")
-                        }
-                        Unit => quote!( #subcommand_var ),
-                        Unnamed(FieldsUnnamed { ref unnamed, .. }) if unnamed.len() == 1 => {
-                            let ty = &unnamed[0];
-                            if override_required {
-                                quote_spanned! { ty.span()=>
-                                    {
-                                        <#ty as clap::Subcommand>::augment_subcommands_for_update(#subcommand_var)
-                                    }
-                                }
-                            } else {
-                                quote_spanned! { ty.span()=>
-                                    {
-                                        <#ty as clap::Subcommand>::augment_subcommands(#subcommand_var)
-                                    }
-                                }
-                            }
-                        }
-                        Unnamed(..) => {
-                            abort!(variant, "non single-typed tuple enums are not supported")
-                        }
-                    };
-
-                    let name = item.cased_name();
+            Kind::Flatten(_) => match variant.fields {
+                Unnamed(FieldsUnnamed { ref unnamed, .. }) if unnamed.len() == 1 => {
+                    let ty = &unnamed[0].ty;
                     let deprecations = if !override_required {
                         item.deprecations()
                     } else {
                         quote!()
                     };
-                    let initial_app_methods = item.initial_top_level_methods();
-                    let final_from_attrs = item.final_top_level_methods();
-                    let subcommand = quote! {
-                        let #app_var = #app_var.subcommand({
-                            #deprecations;
-                            let #subcommand_var = clap::Command::new(#name);
+                    let next_help_heading = item.next_help_heading();
+                    let next_display_order = item.next_display_order();
+                    let subcommand = if override_required {
+                        quote! {
+                            #deprecations
+                            let #app_var = #app_var
+                                #next_help_heading
+                                #next_display_order;
+                            let #app_var = <#ty as clap::Subcommand>::augment_subcommands_for_update(#app_var);
+                        }
+                    } else {
+                        quote! {
+                            #deprecations
+                            let #app_var = #app_var
+                                #next_help_heading
+                                #next_display_order;
+                            let #app_var = <#ty as clap::Subcommand>::augment_subcommands(#app_var);
+                        }
+                    };
+                    Some(subcommand)
+                }
+                _ => abort!(
+                    variant,
+                    "`flatten` is usable only with single-typed tuple variants"
+                ),
+            },
+
+            Kind::Subcommand(_) => {
+                let subcommand_var = Ident::new("__clap_subcommand", Span::call_site());
+                let arg_block = match variant.fields {
+                    Named(_) => {
+                        abort!(variant, "non single-typed tuple enums are not supported")
+                    }
+                    Unit => quote!( #subcommand_var ),
+                    Unnamed(FieldsUnnamed { ref unnamed, .. }) if unnamed.len() == 1 => {
+                        let ty = &unnamed[0].ty;
+                        if override_required {
+                            quote_spanned! { ty.span()=>
+                                {
+                                    <#ty as clap::Subcommand>::augment_subcommands_for_update(#subcommand_var)
+                                }
+                            }
+                        } else {
+                            quote_spanned! { ty.span()=>
+                                {
+                                    <#ty as clap::Subcommand>::augment_subcommands(#subcommand_var)
+                                }
+                            }
+                        }
+                    }
+                    Unnamed(..) => {
+                        abort!(variant, "non single-typed tuple enums are not supported")
+                    }
+                };
+
+                let name = item.cased_name();
+                let deprecations = if !override_required {
+                    item.deprecations()
+                } else {
+                    quote!()
+                };
+                let initial_app_methods = item.initial_top_level_methods();
+                let final_from_attrs = item.final_top_level_methods();
+                let override_methods = if override_required {
+                    quote_spanned! { kind.span()=>
+                        .subcommand_required(false)
+                        .arg_required_else_help(false)
+                    }
+                } else {
+                    quote!()
+                };
+                let subcommand = quote! {
+                    let #app_var = #app_var.subcommand({
+                        #deprecations;
+                        let #subcommand_var = clap::Command::new(#name);
+                        let #subcommand_var = #subcommand_var
+                            .subcommand_required(true)
+                            .arg_required_else_help(true);
+                        let #subcommand_var = #subcommand_var #initial_app_methods;
+                        let #subcommand_var = #arg_block;
+                        #subcommand_var #final_from_attrs #override_methods
+                    });
+                };
+                Some(subcommand)
+            }
+
+            Kind::Command(_) => {
+                let subcommand_var = Ident::new("__clap_subcommand", Span::call_site());
+                let sub_augment = match variant.fields {
+                    Named(ref fields) => {
+                        // Defer to `gen_augment` for adding cmd methods
+                        let fields = collect_args_fields(item, fields)?;
+                        args::gen_augment(&fields, &subcommand_var, item, override_required)?
+                    }
+                    Unit => {
+                        let arg_block = quote!( #subcommand_var );
+                        let initial_app_methods = item.initial_top_level_methods();
+                        let final_from_attrs = item.final_top_level_methods();
+                        quote! {
                             let #subcommand_var = #subcommand_var #initial_app_methods;
                             let #subcommand_var = #arg_block;
-                            let #subcommand_var = #subcommand_var
-                                .subcommand_required(true)
-                                .arg_required_else_help(true);
                             #subcommand_var #final_from_attrs
-                        });
-                    };
-                    Some(subcommand)
-                }
-
-                Kind::Command(_) => {
-                    let subcommand_var = Ident::new("__clap_subcommand", Span::call_site());
-                    let sub_augment = match variant.fields {
-                        Named(ref fields) => {
-                            // Defer to `gen_augment` for adding cmd methods
-                            let fields = fields
-                                .named
-                                .iter()
-                                .map(|field| {
-                                    let item = Item::from_args_field(field, item.casing(), item.env_casing());
-                                    (field, item)
-                                })
-                                .collect::<Vec<_>>();
-                            args::gen_augment(&fields, &subcommand_var, item, override_required)
                         }
-                        Unit => {
-                            let arg_block = quote!( #subcommand_var );
-                            let initial_app_methods = item.initial_top_level_methods();
-                            let final_from_attrs = item.final_top_level_methods();
-                            quote! {
-                                let #subcommand_var = #subcommand_var #initial_app_methods;
-                                let #subcommand_var = #arg_block;
-                                #subcommand_var #final_from_attrs
-                            }
-                        },
-                        Unnamed(FieldsUnnamed { ref unnamed, .. }) if unnamed.len() == 1 => {
-                            let ty = &unnamed[0];
-                            let arg_block = if override_required {
-                                quote_spanned! { ty.span()=>
-                                    {
-                                        <#ty as clap::Args>::augment_args_for_update(#subcommand_var)
-                                    }
+                    }
+                    Unnamed(FieldsUnnamed { ref unnamed, .. }) if unnamed.len() == 1 => {
+                        let ty = &unnamed[0].ty;
+                        let arg_block = if override_required {
+                            quote_spanned! { ty.span()=>
+                                {
+                                    <#ty as clap::Args>::augment_args_for_update(#subcommand_var)
                                 }
-                            } else {
-                                quote_spanned! { ty.span()=>
-                                    {
-                                        <#ty as clap::Args>::augment_args(#subcommand_var)
-                                    }
-                                }
-                            };
-                            let initial_app_methods = item.initial_top_level_methods();
-                            let final_from_attrs = item.final_top_level_methods();
-                            quote! {
-                                let #subcommand_var = #subcommand_var #initial_app_methods;
-                                let #subcommand_var = #arg_block;
-                                #subcommand_var #final_from_attrs
                             }
+                        } else {
+                            quote_spanned! { ty.span()=>
+                                {
+                                    <#ty as clap::Args>::augment_args(#subcommand_var)
+                                }
+                            }
+                        };
+                        let initial_app_methods = item.initial_top_level_methods();
+                        let final_from_attrs = item.final_top_level_methods();
+                        quote! {
+                            let #subcommand_var = #subcommand_var #initial_app_methods;
+                            let #subcommand_var = #arg_block;
+                            #subcommand_var #final_from_attrs
                         }
-                        Unnamed(..) => {
-                            abort!(variant, "non single-typed tuple enums are not supported")
-                        }
-                    };
+                    }
+                    Unnamed(..) => {
+                        abort!(variant, "non single-typed tuple enums are not supported")
+                    }
+                };
 
-                    let deprecations = if !override_required {
-                        item.deprecations()
-                    } else {
-                        quote!()
-                    };
-                    let name = item.cased_name();
-                    let subcommand = quote! {
-                        let #app_var = #app_var.subcommand({
-                            #deprecations
-                            let #subcommand_var = clap::Command::new(#name);
-                            #sub_augment
-                        });
-                    };
-                    Some(subcommand)
-                }
+                let deprecations = if !override_required {
+                    item.deprecations()
+                } else {
+                    quote!()
+                };
+                let name = item.cased_name();
+                let subcommand = quote! {
+                    let #app_var = #app_var.subcommand({
+                        #deprecations
+                        let #subcommand_var = clap::Command::new(#name);
+                        #sub_augment
+                    });
+                };
+                Some(subcommand)
             }
-        })
-        .collect();
+        };
+        subcommands.push(genned);
+    }
 
     let deprecations = if !override_required {
         parent_item.deprecations()
@@ -338,16 +348,16 @@ fn gen_augment(
     };
     let initial_app_methods = parent_item.initial_top_level_methods();
     let final_app_methods = parent_item.final_top_level_methods();
-    quote! {
+    Ok(quote! {
         #deprecations;
         let #app_var = #app_var #initial_app_methods;
         #( #subcommands )*;
         #app_var #final_app_methods
-    }
+    })
 }
 
-fn gen_has_subcommand(variants: &[(&Variant, Item)]) -> TokenStream {
-    use syn::Fields::*;
+fn gen_has_subcommand(variants: &[(&Variant, Item)]) -> Result<TokenStream, syn::Error> {
+    use syn::Fields::Unnamed;
 
     let mut ext_subcmd = false;
 
@@ -382,20 +392,21 @@ fn gen_has_subcommand(variants: &[(&Variant, Item)]) -> TokenStream {
         .iter()
         .map(|(variant, _attrs)| match variant.fields {
             Unnamed(ref fields) if fields.unnamed.len() == 1 => {
-                let ty = &fields.unnamed[0];
-                quote! {
+                let ty = &fields.unnamed[0].ty;
+                Ok(quote! {
                     if <#ty as clap::Subcommand>::has_subcommand(__clap_name) {
                         return true;
                     }
-                }
+                })
             }
             _ => abort!(
                 variant,
                 "`flatten` is usable only with single-typed tuple variants"
             ),
-        });
+        })
+        .collect::<Result<Vec<_>, syn::Error>>()?;
 
-    if ext_subcmd {
+    let genned = if ext_subcmd {
         quote! { true }
     } else {
         quote! {
@@ -405,111 +416,106 @@ fn gen_has_subcommand(variants: &[(&Variant, Item)]) -> TokenStream {
 
             false
         }
-    }
+    };
+    Ok(genned)
 }
 
-fn gen_from_arg_matches(variants: &[(&Variant, Item)]) -> TokenStream {
-    use syn::Fields::*;
-
-    let mut ext_subcmd = None;
+fn gen_from_arg_matches(variants: &[(&Variant, Item)]) -> Result<TokenStream, syn::Error> {
+    use syn::Fields::{Named, Unit, Unnamed};
 
     let subcommand_name_var = format_ident!("__clap_name");
     let sub_arg_matches_var = format_ident!("__clap_arg_matches");
-    let (flatten_variants, variants): (Vec<_>, Vec<_>) = variants
-        .iter()
-        .filter_map(|(variant, item)| {
-            let kind = item.kind();
-            match &*kind {
-                Kind::Skip(_, _) | Kind::Arg(_) | Kind::FromGlobal(_) | Kind::Value => None,
 
-                Kind::ExternalSubcommand => {
-                    if ext_subcmd.is_some() {
-                        abort!(
-                            item.kind().span(),
-                            "Only one variant can be marked with `external_subcommand`, \
+    let mut ext_subcmd = None;
+    let mut flatten_variants = Vec::new();
+    let mut unflatten_variants = Vec::new();
+    for (variant, item) in variants {
+        let kind = item.kind();
+        match &*kind {
+            Kind::Skip(_, _) | Kind::Arg(_) | Kind::FromGlobal(_) | Kind::Value => {}
+
+            Kind::ExternalSubcommand => {
+                if ext_subcmd.is_some() {
+                    abort!(
+                        item.kind().span(),
+                        "Only one variant can be marked with `external_subcommand`, \
                          this is the second"
-                        );
-                    }
+                    );
+                }
 
-                    let ty = match variant.fields {
-                        Unnamed(ref fields) if fields.unnamed.len() == 1 => &fields.unnamed[0].ty,
+                let ty = match variant.fields {
+                    Unnamed(ref fields) if fields.unnamed.len() == 1 => &fields.unnamed[0].ty,
 
-                        _ => abort!(
-                            variant,
-                            "The enum variant marked with `external_subcommand` must be \
+                    _ => abort!(
+                        variant,
+                        "The enum variant marked with `external_subcommand` must be \
                          a single-typed tuple, and the type must be either `Vec<String>` \
                          or `Vec<OsString>`."
-                        ),
-                    };
+                    ),
+                };
 
-                    let (span, str_ty) = match subty_if_name(ty, "Vec") {
-                        Some(subty) => {
-                            if is_simple_ty(subty, "String") {
-                                (subty.span(), quote!(::std::string::String))
-                            } else if is_simple_ty(subty, "OsString") {
-                                (subty.span(), quote!(::std::ffi::OsString))
-                            } else {
-                                abort!(
-                                    ty.span(),
-                                    "The type must be either `Vec<String>` or `Vec<OsString>` \
+                let (span, str_ty) = match subty_if_name(ty, "Vec") {
+                    Some(subty) => {
+                        if is_simple_ty(subty, "String") {
+                            (subty.span(), quote!(::std::string::String))
+                        } else if is_simple_ty(subty, "OsString") {
+                            (subty.span(), quote!(::std::ffi::OsString))
+                        } else {
+                            abort!(
+                                ty.span(),
+                                "The type must be either `Vec<String>` or `Vec<OsString>` \
                                  to be used with `external_subcommand`."
-                                );
-                            }
+                            );
                         }
+                    }
 
-                        None => abort!(
-                            ty.span(),
-                            "The type must be either `Vec<String>` or `Vec<OsString>` \
+                    None => abort!(
+                        ty.span(),
+                        "The type must be either `Vec<String>` or `Vec<OsString>` \
                          to be used with `external_subcommand`."
-                        ),
-                    };
+                    ),
+                };
 
-                    ext_subcmd = Some((span, &variant.ident, str_ty));
-                    None
-                }
-                Kind::Flatten(_) | Kind::Subcommand(_) | Kind::Command(_) => Some((variant, item)),
+                ext_subcmd = Some((span, &variant.ident, str_ty));
             }
-        })
-        .partition(|(_, item)| {
-            let kind = item.kind();
-            matches!(&*kind, Kind::Flatten(_))
-        });
+            Kind::Flatten(_) | Kind::Subcommand(_) | Kind::Command(_) => {
+                if matches!(&*item.kind(), Kind::Flatten(_)) {
+                    flatten_variants.push((variant, item));
+                } else {
+                    unflatten_variants.push((variant, item));
+                }
+            }
+        }
+    }
 
-    let subcommands = variants.iter().map(|(variant, item)| {
+    let subcommands = unflatten_variants.iter().map(|(variant, item)| {
         let sub_name = item.cased_name();
         let variant_name = &variant.ident;
         let constructor_block = match variant.fields {
             Named(ref fields) => {
-                let fields = fields
-                    .named
-                    .iter()
-                    .map(|field| {
-                        let item = Item::from_args_field(field, item.casing(), item.env_casing());
-                        (field, item)
-                    })
-                    .collect::<Vec<_>>();
-                args::gen_constructor(&fields)
+                let fields = collect_args_fields(item, fields)?;
+                args::gen_constructor(&fields)?
             },
             Unit => quote!(),
             Unnamed(ref fields) if fields.unnamed.len() == 1 => {
-                let ty = &fields.unnamed[0];
+                let ty = &fields.unnamed[0].ty;
                 quote!( ( <#ty as clap::FromArgMatches>::from_arg_matches_mut(__clap_arg_matches)? ) )
             }
             Unnamed(..) => abort_call_site!("{}: tuple enums are not supported", variant.ident),
         };
 
-        quote! {
+        Ok(quote! {
             if #subcommand_name_var == #sub_name && !#sub_arg_matches_var.contains_id("") {
                 return ::std::result::Result::Ok(Self :: #variant_name #constructor_block)
             }
-        }
-    });
+        })
+    }).collect::<Result<Vec<_>, syn::Error>>()?;
     let child_subcommands = flatten_variants.iter().map(|(variant, _attrs)| {
         let variant_name = &variant.ident;
         match variant.fields {
             Unnamed(ref fields) if fields.unnamed.len() == 1 => {
-                let ty = &fields.unnamed[0];
-                quote! {
+                let ty = &fields.unnamed[0].ty;
+                Ok(quote! {
                     if __clap_arg_matches
                         .subcommand_name()
                         .map(|__clap_name| <#ty as clap::Subcommand>::has_subcommand(__clap_name))
@@ -518,14 +524,14 @@ fn gen_from_arg_matches(variants: &[(&Variant, Item)]) -> TokenStream {
                         let __clap_res = <#ty as clap::FromArgMatches>::from_arg_matches_mut(__clap_arg_matches)?;
                         return ::std::result::Result::Ok(Self :: #variant_name (__clap_res));
                     }
-                }
+                })
             }
             _ => abort!(
                 variant,
                 "`flatten` is usable only with single-typed tuple variants"
             ),
         }
-    });
+    }).collect::<Result<Vec<_>, syn::Error>>()?;
 
     let wildcard = match ext_subcmd {
         Some((span, var_name, str_ty)) => quote_spanned! { span=>
@@ -542,12 +548,12 @@ fn gen_from_arg_matches(variants: &[(&Variant, Item)]) -> TokenStream {
         },
 
         None => quote! {
-            ::std::result::Result::Err(clap::Error::raw(clap::error::ErrorKind::InvalidSubcommand, format!("The subcommand '{}' wasn't recognized", #subcommand_name_var)))
+            ::std::result::Result::Err(clap::Error::raw(clap::error::ErrorKind::InvalidSubcommand, format!("the subcommand '{}' wasn't recognized", #subcommand_name_var)))
         },
     };
 
     let raw_deprecated = args::raw_deprecated();
-    quote! {
+    Ok(quote! {
         fn from_arg_matches_mut(__clap_arg_matches: &mut clap::ArgMatches) -> ::std::result::Result<Self, clap::Error> {
             #raw_deprecated
 
@@ -559,14 +565,14 @@ fn gen_from_arg_matches(variants: &[(&Variant, Item)]) -> TokenStream {
 
                 #wildcard
             } else {
-                ::std::result::Result::Err(clap::Error::raw(clap::error::ErrorKind::MissingSubcommand, "A subcommand is required but one was not provided."))
+                ::std::result::Result::Err(clap::Error::raw(clap::error::ErrorKind::MissingSubcommand, "a subcommand is required but one was not provided"))
             }
         }
-    }
+    })
 }
 
-fn gen_update_from_arg_matches(variants: &[(&Variant, Item)]) -> TokenStream {
-    use syn::Fields::*;
+fn gen_update_from_arg_matches(variants: &[(&Variant, Item)]) -> Result<TokenStream, syn::Error> {
+    use syn::Fields::{Named, Unit, Unnamed};
 
     let (flatten, variants): (Vec<_>, Vec<_>) = variants
         .iter()
@@ -595,15 +601,8 @@ fn gen_update_from_arg_matches(variants: &[(&Variant, Item)]) -> TokenStream {
                 let field_names = fields.named.iter().map(|field| {
                     field.ident.as_ref().unwrap()
                 }).collect::<Vec<_>>();
-                let fields = fields
-                    .named
-                    .iter()
-                    .map(|field| {
-                        let item = Item::from_args_field(field, item.casing(), item.env_casing());
-                        (field, item)
-                    })
-                    .collect::<Vec<_>>();
-                let update = args::gen_updater(&fields, false);
+                let fields = collect_args_fields(item, fields)?;
+                let update = args::gen_updater(&fields, false)?;
                 (quote!( { #( #field_names, )* }), quote!( { #update } ))
             }
             Unit => (quote!(), quote!({})),
@@ -622,38 +621,38 @@ fn gen_update_from_arg_matches(variants: &[(&Variant, Item)]) -> TokenStream {
             }
         };
 
-        quote! {
+        Ok(quote! {
             Self :: #variant_name #pattern if #sub_name == __clap_name => {
                 let (_, mut __clap_arg_sub_matches) = __clap_arg_matches.remove_subcommand().unwrap();
                 let __clap_arg_matches = &mut __clap_arg_sub_matches;
                 #updater
             }
-        }
-    });
+        })
+    }).collect::<Result<Vec<_>, _>>()?;
 
     let child_subcommands = flatten.iter().map(|(variant, _attrs)| {
         let variant_name = &variant.ident;
         match variant.fields {
             Unnamed(ref fields) if fields.unnamed.len() == 1 => {
-                let ty = &fields.unnamed[0];
-                quote! {
+                let ty = &fields.unnamed[0].ty;
+                Ok(quote! {
                     if <#ty as clap::Subcommand>::has_subcommand(__clap_name) {
                         if let Self :: #variant_name (child) = s {
                             <#ty as clap::FromArgMatches>::update_from_arg_matches_mut(child, __clap_arg_matches)?;
                             return ::std::result::Result::Ok(());
                         }
                     }
-                }
+                })
             }
             _ => abort!(
                 variant,
                 "`flatten` is usable only with single-typed tuple variants"
             ),
         }
-    });
+    }).collect::<Result<Vec<_>, _>>()?;
 
     let raw_deprecated = args::raw_deprecated();
-    quote! {
+    Ok(quote! {
         fn update_from_arg_matches_mut<'b>(
             &mut self,
             __clap_arg_matches: &mut clap::ArgMatches,
@@ -671,5 +670,5 @@ fn gen_update_from_arg_matches(variants: &[(&Variant, Item)]) -> TokenStream {
             }
             ::std::result::Result::Ok(())
         }
-    }
+    })
 }
